@@ -47,7 +47,6 @@
 #include <libxml/xmlmemory.h>
 
 #include <libsoup/soup.h>
-#include <libsoup/soup-xmlrpc-message.h>
 
 
 #define APPLET_REQUIREMENTS \
@@ -461,10 +460,8 @@ bugzilla_search_for_package (gpointer key, gpointer value, const char *package)
 char *
 bugzilla_parse_response (SoupMessage *msg, GError **err)
 {
-	SoupXmlrpcResponse *response;
-        SoupXmlrpcValue *value;
-	long bugid = 0;
-	int debug = 0;
+	GValue value;
+	int bugid = 0;
 	char *url;
 
 	g_return_val_if_fail ((err == NULL || *err == NULL), NULL);
@@ -475,88 +472,39 @@ bugzilla_parse_response (SoupMessage *msg, GError **err)
 		return NULL;
 	}
 
-	response = soup_xmlrpc_message_parse_response (SOUP_XMLRPC_MESSAGE (msg));
-	if (!response) {
+	if (!soup_xmlrpc_parse_method_response (msg->response_body->data,
+						msg->response_body->length,
+						&value, err))
+		return NULL;
+
+	if (G_VALUE_HOLDS_INT (&value))
+		bugid = g_value_get_int (&value);
+	else if (G_VALUE_HOLDS_STRING (&value))
+		url = g_value_dup_string (&value);
+	else {
+		g_value_unset (&value);
 		g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_RECV_PARSE_FAILED, 
-		             _("Unable to parse XML-RPC Response")); 
+			     _("Unable to parse XML-RPC Response\n\n%s"),
+			     msg->response_body->data);
 		return NULL;
 	}
+	g_value_unset (&value);
 
-	/* check to see if the XMLRPC response was a <fault> */
-	if (soup_xmlrpc_response_is_fault (response)) {
-		SoupXmlrpcValue *faultCode = NULL;
-		SoupXmlrpcValue *faultString = NULL;
-		GHashTable *fault = NULL;
-		gchar *errormsg = NULL;
-		long errorcode = -1;
-
-		value = soup_xmlrpc_response_get_value (response);
-		if (!value) {
-			debug = 1;
-			goto parse_error;
-		}
-
-		/* get the struct representing the fault */
-		if (!soup_xmlrpc_value_get_struct (value, &fault)) {
-			debug = 2;
-			goto parse_error;
-		}
-
-		/* get the integer representing the fault code */
-		faultCode = g_hash_table_lookup (fault, "faultCode");
-		if (faultCode == NULL || !soup_xmlrpc_value_get_int (faultCode, &errorcode)) {
-			debug = 3;
-			goto parse_error;
-		}
-		
-		/* get the string representing the fault string */
-		faultString = g_hash_table_lookup (fault, "faultString");
-		if (faultString == NULL || !soup_xmlrpc_value_get_string (faultString, &errormsg)) {
-			debug = 4;
-			goto parse_error;
-		}
-			
-		/* send back a BUGZILLA_ERROR_FAULT, using the errorcode and errorstring to
-		 * construct the GError message  */
-		g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_RECV_FAULT, 
-		             "%ld:%s", errorcode, errormsg);
-		return NULL;
-
-	}
-
-	value = soup_xmlrpc_response_get_value (response);
-	if (!value) {
-		debug = 5;
-		goto parse_error;
-	}
-
-	if (!soup_xmlrpc_value_get_int (value, &bugid)) {
-		if (!soup_xmlrpc_value_get_string (value, &url)) {
-			debug = 6;
-			goto parse_error;
-		}
-	}	
-
-	/* whew, everything checked out, send back the bug id */
-	g_object_unref (response);
-	return bugid ? g_strdup_printf ("%ld", bugid) : url;
-
-parse_error:
-	g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_RECV_PARSE_FAILED, 
-	             _("Unable to parse XML-RPC Response\n\n%d\n\n%s"),
-	             debug, soup_xmlrpc_response_to_string (response));
-	g_object_unref (response);
-	return NULL;
+	return bugid ? g_strdup_printf ("%d", bugid) : url;
 }
 
-SoupXmlrpcMessage*
+SoupMessage*
 bugzilla_create_report (BugzillaApplication *app, int type, GnomeVersionInfo *gnome_version,
 		        const char *username, const char *title, const char *text,
 		        GtkBuilder *ui, const char *minidump_file, GError **err)
 {
-	SoupXmlrpcMessage *message;
+	SoupMessage *message;
+	const char *uri;
+	GHashTable *report;
 	char *user_agent;
 	char *os_version;
+	gchar *crt;
+    	GString *rv;
 
 	g_return_val_if_fail (app != NULL, NULL);
 	g_return_val_if_fail (gnome_version != NULL, NULL);
@@ -578,70 +526,42 @@ bugzilla_create_report (BugzillaApplication *app, int type, GnomeVersionInfo *gn
 		return NULL;
 	}
 
-	if (minidump_file)
-		//message = soup_xmlrpc_message_new ("http://localhost/breakpad/xmlrpc.py");
-		message = soup_xmlrpc_message_new ("http://socorro.gnome.org/collect.py");
-	else 
-		message = soup_xmlrpc_message_new ("http://bugzilla.gnome.org/bugbuddy.cgi");
+	report = soup_value_hash_new ();
 
-	if (message == NULL) {
-		g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_SEND_ERROR,
-		             _("Unable to create XML-RPC message."));
-		return NULL;
-	}
-
-	user_agent = g_strdup_printf ("Bug-Buddy: %s", VERSION);
-	soup_message_add_header (SOUP_MESSAGE(message)->request_headers, "User-Agent", user_agent);
-	g_free (user_agent);
-
-	soup_xmlrpc_message_start_call (message, "BugBuddy.createBug");
-	soup_xmlrpc_message_start_param (message);
-	soup_xmlrpc_message_start_struct (message);
-
-	soup_xmlrpc_message_start_member (message, "version");
-	soup_xmlrpc_message_write_string (message, app->version ? app->version : "unspecified");
-	soup_xmlrpc_message_end_member (message);
-
-	soup_xmlrpc_message_start_member (message, "product");
-	soup_xmlrpc_message_write_string (message, app->product);
-	soup_xmlrpc_message_end_member (message);
-
-	soup_xmlrpc_message_start_member (message, "component");
-	soup_xmlrpc_message_write_string (message, app->component);
-	soup_xmlrpc_message_end_member (message);
-
-	soup_xmlrpc_message_start_member (message, "gnome_version");
-	soup_xmlrpc_message_write_string (message, gnome_version->gnome_platform);
-	soup_xmlrpc_message_end_member (message);
-
-	soup_xmlrpc_message_start_member (message, "reporter");
-	soup_xmlrpc_message_write_string (message, username);
-	soup_xmlrpc_message_end_member (message);
+	soup_value_hash_insert (report, "version", G_TYPE_STRING,
+				app->version ? app->version : "unspecified");
+	soup_value_hash_insert (report, "product", G_TYPE_STRING, app->product);
+	soup_value_hash_insert (report, "component", G_TYPE_STRING, app->component);
+	soup_value_hash_insert (report, "gnome_version", G_TYPE_STRING,
+				gnome_version->gnome_platform);
+	soup_value_hash_insert (report, "reporter", G_TYPE_STRING, username);
 
 	os_version = get_distro_name ();
-	soup_xmlrpc_message_start_member (message, "os_version");
-	soup_xmlrpc_message_write_string (message, os_version);
-	soup_xmlrpc_message_end_member (message);
+	soup_value_hash_insert (report, "os_version", G_TYPE_STRING, os_version);
 	g_free (os_version);
 
 	if (type == BUG_TYPE_CRASH) {
-		soup_xmlrpc_message_start_member (message, "priority");
-		soup_xmlrpc_message_write_string (message, "High");
-		soup_xmlrpc_message_end_member (message);
-
-		soup_xmlrpc_message_start_member (message, "bug_severity");
-		soup_xmlrpc_message_write_string (message, "critical");
-		soup_xmlrpc_message_end_member (message);
+		soup_value_hash_insert (report, "priority", G_TYPE_STRING, "High");
+		soup_value_hash_insert (report, "bug_severity", G_TYPE_STRING, "critical");
 	}
 
-	soup_xmlrpc_message_start_member (message, "short_desc");
-	soup_xmlrpc_message_write_string (message, title); 
-	soup_xmlrpc_message_end_member (message);
+	soup_value_hash_insert (report, "short_desc", G_TYPE_STRING, title);
 
-	soup_xmlrpc_message_start_member (message, "comment");
-	soup_xmlrpc_message_write_string (message, text); 
-	soup_xmlrpc_message_end_member (message);
+	/* Skip UTF-8 control chars that are not valid in XML.*/
+    	rv = g_string_new (NULL);
+    	crt = text;
+    	while (*crt) {
+		gchar *next = g_utf8_next_char (crt);
+		gunichar uni = g_utf8_get_char (crt);
 
+		if (!g_unichar_iscntrl (uni) || (uni == '\n') || (uni == '\t'))
+	    		g_string_append_len (rv, crt, next - crt);
+
+		crt = next;
+    	}
+	
+	soup_value_hash_insert (report, "comment", G_TYPE_STRING, rv->str);
+    	g_string_free (rv, TRUE);
 
 	if (minidump_file) {
 		gchar *minidumpbuf;
@@ -659,31 +579,54 @@ bugzilla_create_report (BugzillaApplication *app, int type, GnomeVersionInfo *gn
 		if (base64) {	
 			gchar *basename;
 			gchar *id;
+			GByteArray *ba;
 
 			basename = g_path_get_basename (minidump_file);
 			id = g_strndup (basename, strlen (basename) - strlen (".dmp"));
 			
-			soup_xmlrpc_message_start_member (message, "minidump-id");
-			soup_xmlrpc_message_write_string (message, id); 
-			soup_xmlrpc_message_end_member (message);
-			
-			soup_xmlrpc_message_start_member (message, "minidump");
-			soup_xmlrpc_message_write_base64 (message, base64, strlen(base64)); 
-			soup_xmlrpc_message_end_member (message);
+			soup_value_hash_insert (report, "minidump-id",
+						G_TYPE_STRING, id);
+
+			/* FIXME: This is broken; for some reason
+			 * bug-buddy is base64-encoding the minidump
+			 * *twice*. (Once itself, and once by asking
+			 * libsoup to do it.) Fix this after verifying
+			 * that the server side can deal with the
+			 * fix...
+			 */
+			ba = g_byte_array_sized_new (strlen (base64));
+			g_byte_array_append (ba, (guchar *)base64, strlen (base64));
+			soup_value_hash_insert (report, "minidump",
+						SOUP_TYPE_BYTE_ARRAY, ba);
+			g_byte_array_free (ba, TRUE);
 			g_free (base64);
 			g_free (basename);
 			g_free (id);
 		}
 	}
 
+	if (minidump_file)
+		//uri = "http://localhost/breakpad/xmlrpc.py";
+		uri = "http://socorro.gnome.org/collect.py";
+	else 
+		uri = "http://bugzilla.gnome.org/bugbuddy.cgi";
 
+	message = soup_xmlrpc_request_new (uri, "BugBuddy.createBug",
+					   G_TYPE_HASH_TABLE, report,
+					   G_TYPE_INVALID);
+	g_hash_table_destroy (report);
+	if (message == NULL) {
+		g_set_error (err, BUGZILLA_ERROR, BUGZILLA_ERROR_SEND_ERROR,
+		             _("Unable to create XML-RPC message."));
+		return NULL;
+	}
 
-	soup_xmlrpc_message_end_param (message);
-	soup_xmlrpc_message_end_struct (message);
-
-	soup_xmlrpc_message_end_call (message);
+	/* FIXME: wrong User-Agent syntax. Should be "Bug-Buddy (VERSION)" */
+	user_agent = g_strdup_printf ("Bug-Buddy: %s", VERSION);
+	soup_message_headers_append (message->request_headers,
+				     "User-Agent", user_agent);
+	g_free (user_agent);
 	
 	return message;
-
 }
 	
