@@ -26,9 +26,9 @@
 #include <libxml/parser.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtktreemodel.h>
-#include <glade/glade.h>
+#include <gtk/gtkbuilder.h>
 
-#include <libsoup/soup-xmlrpc-message.h>
+#include <libsoup/soup-message.h>
 
 extern GQuark bugzilla_error_quark (void);
 #define BUGZILLA_ERROR bugzilla_error_quark ()
@@ -36,7 +36,6 @@ extern GQuark bugzilla_error_quark (void);
 typedef enum {
 	BUGZILLA_ERROR_RECV_BAD_STATUS,
 	BUGZILLA_ERROR_RECV_PARSE_FAILED,
-	BUGZILLA_ERROR_RECV_FAULT,
 	BUGZILLA_ERROR_SEND_NOTSUPPORTED_APP,
 	BUGZILLA_ERROR_SEND_ERROR
 } BugzillaError;
@@ -74,17 +73,18 @@ typedef struct _GnomeVersionInfo GnomeVersionInfo;
 	
 
 GHashTable*        load_applications       (void);
-int                bugzilla_parse_response (SoupMessage *msg,
+char*              bugzilla_parse_response (SoupMessage *msg,
                                             GError **err);
 gboolean	   bugzilla_search_for_package (gpointer key, gpointer value, const char *package);
 
-SoupXmlrpcMessage* bugzilla_create_report  (BugzillaApplication *app, 
+SoupMessage*       bugzilla_create_report  (BugzillaApplication *app, 
 					    int type,
                                             GnomeVersionInfo *gnome_version,
                                             const char *username,
                                             const char *title,
                                             const char *text,
-                                            GladeXML *xml,
+                                            GtkBuilder *ui,
+					    const char *minidump_file,
                                             GError **err);
 
 #endif /* __BUGZILLA_H__ */
